@@ -103,6 +103,33 @@ Dcmtk_rt_study::image_load ()
         return;
     }
 
+    /* Check that every slice image in series have the same dimention */
+    std::vector< std::array< size_t, 3 > > dimParams; 
+    for (Dcmtk_file_list::iterator it = flist->begin(); it != flist->end(); ++it) {
+      const Dcmtk_file* df = (*it).get();
+      DicomImage di (df->get_dataset(), EXS_Unknown);
+      Volume_header vh (df->get_volume_header());
+      plm_long *dim = vh.get_dim();
+    
+      unsigned long length = di.getOutputDataSize(8);
+      dimParams.push_back({ size_t(length), size_t(dim[0]), size_t(dim[1])});
+    }
+    for ( size_t i = 1; i < dimParams.size(); ++i) {
+      std::array< size_t, 3 >& sliceDim = dimParams[i];
+      if (sliceDim[0] != dimParams[0][0] 
+        || sliceDim[1] != dimParams[0][1] 
+        || sliceDim[2] != dimParams[0][2]) {
+        
+        print_and_exit ("Error. Wrong slice dimention, comparing to the previous ones.\n"
+          "Possibly data corrupted or partially downloaded! Please, check the data!\n"
+          "Corrupted slice index is %u\n"
+          "Correct slice data must be (%d vs. %d x %d)\n"  
+          "Corrupted slice data is (%d vs. %d x %d).\n", i, 
+          dimParams[0][0], dimParams[0][1], dimParams[0][2],
+          sliceDim[0], sliceDim[1], sliceDim[2]);
+      }
+    }
+
     /* Get first slice */
     const Dcmtk_file* df = (*flist->begin()).get();
     
@@ -448,7 +475,6 @@ Dcmtk_rt_study::image_load ()
             }
         }
 #else
-        df->get_dataset()->chooseRepresentation (EXS_LittleEndianExplicit, nullptr);
         rc = 0;
         if (bits_alloc == 8) {
             rc = df->get_uint8_array (DCM_PixelData, &pixel_data_8, &length);
@@ -458,9 +484,16 @@ Dcmtk_rt_study::image_load ()
 	if (!rc) {
 	    print_and_exit ("Oops.  Error reading pixel data.  Punting.\n");
 	}
+
+        /* GCS 2022-04-27.  Some GE scans have OB type with twice the length
+           but are actually OW.  Why??  */
+        if (bits_alloc == 16 && length == 2 * dim[0] * dim[1]) {
+            length = dim[0] * dim[1];
+        }
+        
 	if (((long) length) != dim[0] * dim[1]) {
 	    print_and_exit ("Oops.  Dicom image had wrong length "
-		"(%d vs. %d x %d).\n", length, dim[0], dim[1]);
+		"(%d vs. %d x %d = %d).\n", length, dim[0], dim[1], dim[0] * dim[1]);
 	}
 #endif /* USE_DICOMIMAGE */
 
@@ -520,9 +553,10 @@ dcmtk_save_slice (const Rt_study_metadata::Pointer rsm, Dcmtk_slice_data *dsd)
         image_metadata = rsm->get_image_metadata ();
     }
 
-    /* Patient, and General Study modules */
+    /* Add entries for common modules */
     Dcmtk_module::set_patient (dataset, image_metadata);
     Dcmtk_module::set_general_study (dataset, rsm);
+    Dcmtk_module::set_sop_common (dataset);
     
     /* Get modality */
     std::string modality = "CT";
@@ -622,6 +656,7 @@ dcmtk_save_slice (const Rt_study_metadata::Pointer rsm, Dcmtk_slice_data *dsd)
             DCM_SOPClassUID, UID_CTImageStorage);
     }
     dataset->putAndInsertString (DCM_SOPInstanceUID, dsd->slice_uid);
+#if defined (commentout)
     if (image_metadata->get_metadata (DCM_InstanceCreationDate) != "") {
         dataset->putAndInsertOFStringArray(DCM_InstanceCreationDate, 
             image_metadata->get_metadata(DCM_InstanceCreationDate).c_str());
@@ -636,7 +671,15 @@ dcmtk_save_slice (const Rt_study_metadata::Pointer rsm, Dcmtk_slice_data *dsd)
         dataset->putAndInsertOFStringArray(DCM_InstanceCreationTime, 
             rsm->get_study_time());
     }
-
+#endif
+    /* GCS 2022-02-10.  The above is inconsistent with what RTSS and RTDOSE are 
+       doing.  It is hard to see which approach is better, but at least they 
+       should all do the same thing. */
+    dataset->putAndInsertOFStringArray(DCM_InstanceCreationDate, 
+        rsm->get_study_date());
+    dataset->putAndInsertOFStringArray(DCM_InstanceCreationTime, 
+        rsm->get_study_time());
+    
     /* Write the output file */
     OFCondition status = fileformat.saveFile (dsd->fn.c_str(), 
         EXS_LittleEndianExplicit);
