@@ -10,6 +10,7 @@
 
 #include "dcmtk_file.h"
 #include "dcmtk_metadata.h"
+#include "dcmtk_module.h"
 #include "dcmtk_rt_study.h"
 #include "dcmtk_rt_study_p.h"
 #include "dcmtk_series.h"
@@ -72,10 +73,19 @@ Dcmtk_rt_study::rtss_load (void)
         print_and_exit ("Oops.\n");
     }
 
-    /* FIX: load metadata such as patient name, etc. */
+    /* Store metadata such as patient name, etc. */
     if (d_ptr->rt_study_metadata) {
         d_ptr->rt_study_metadata->set_rtstruct_instance_uid (
 	    ds_rtss->get_cstr (DCM_SOPInstanceUID));
+
+        const Dcmtk_file::Pointer& df = ds_rtss->get_dcmtk_file();
+        Metadata::Pointer& study_metadata
+            = d_ptr->rt_study_metadata->get_study_metadata ();
+        dcmtk_copy_into_metadata (study_metadata, df, DCM_PatientName);
+        dcmtk_copy_into_metadata (study_metadata, df, DCM_PatientID);
+        dcmtk_copy_into_metadata (study_metadata, df, DCM_PatientSex);
+        dcmtk_copy_into_metadata (study_metadata, df, DCM_PatientPosition);
+        dcmtk_copy_into_metadata (study_metadata, df, DCM_StudyID);
     }
 
     /* ReferencedFrameOfReferenceSequence */
@@ -264,6 +274,11 @@ Dcmtk_rt_study::rtss_save (const char *dicom_dir)
     DcmFileFormat fileformat;
     DcmDataset *dataset = fileformat.getDataset();
 
+    /* Add entries for common modules */
+    Dcmtk_module::set_sop_common (dataset);
+    
+    /* GCS FIX, remove below code, use common modules instead */
+
     /* ----------------------------------------------------------------- */
     /*     Part 1  -- General header                                     */
     /* ----------------------------------------------------------------- */
@@ -300,12 +315,6 @@ Dcmtk_rt_study::rtss_save (const char *dicom_dir)
     dataset->putAndInsertString (DCM_SoftwareVersions,
         PLASTIMATCH_VERSION_STRING);
 
-#if defined (commentout)
-    /* GCS FIX */
-    /* PatientPosition */
-    // gf->InsertValEntry (xxx, 0x0018, 0x5100);
-#endif
-
     dataset->putAndInsertString (DCM_StudyInstanceUID, 
         d_ptr->rt_study_metadata->get_study_uid().c_str());
     dataset->putAndInsertString (DCM_SeriesInstanceUID, 
