@@ -8,12 +8,6 @@
 #include <string.h>
 #include <stdlib.h>
 
-class Compare_parms {
-public:
-    std::string img_in_1_fn;
-    std::string img_in_2_fn;
-};
-
 void do_command_compare (int argc, char *argv[]);
 
 #endif
