@@ -9,8 +9,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include "itkImage.h"
-#include "itkImageFileReader.h"
-#include "itkImageFileWriter.h"
 #include "itkPoint.h"
 #include "itkPointSet.h"
 #include "itkThinPlateSplineKernelTransform.h"
