@@ -7,6 +7,23 @@
 #include "YK16GrayImage.h"
 
 #include "plm_image.h"
+
+/// This workaround prevents a compilation fail with ITK-5.3
+/// Something defines POSIX in preprocessor, and it conflicts
+/// with ITK KWSys POSIX enumeration constant
+#if (ITK_VERSION_MAJOR == 5) && (ITK_VERSION_MINOR > 2) && defined(POSIX)
+#define PLMPOSIX_TMP (POSIX)
+#undef POSIX
+#endif
+
+#include "itkImageFileReader.h"
+
+#ifdef PLMPOSIX_TMP
+#define POSIX (PLMPOSIX_TMP)
+#undef PLMPOSIX_TMP
+#endif
+
+#include "itkImageFileWriter.h"
 #include "rt_study_metadata.h"
 #include "gamma_dose_comparison.h"
 
@@ -31,8 +48,6 @@
 #include "dcmtk/dcmdata/dcdeftag.h"
 
 #include <QDataStream>
-#include "itkImageFileReader.h"
-#include "itkImageFileWriter.h"
 #include "itkImageSliceIteratorWithIndex.h"
 #include "itkFlipImageFilter.h"
 
